#import "AppcarrySdkPlugin.h"
#import <objc/message.h>


typedef void (*bypassDidFinishLaunchingWithOption)(id, SEL, NSInteger);
typedef void (*bypassDisableAdvertisingIdentifier)(id, SEL, BOOL);
typedef void (*bypassWaitForATTUserAuthorization)(id, SEL, NSTimeInterval);

@implementation AppcarrySdkPlugin{
    FlutterEventChannel *_eventChannel;
    // AppcarryStreamHandler *_streamHandler;
}

static NSMutableArray* _callbackById;
static FlutterMethodChannel* _callbackChannel;
static BOOL _gcdCallback = false;
static BOOL _oaoaCallback = false;
static BOOL _udpCallback = false;
// static BOOL _isPushNotificationEnabled = false;
// static BOOL _isSandboxEnabled = false;
static BOOL _isSKADEnabled = false;

+ (FlutterMethodChannel*)callbackChannel{
    return _callbackChannel;
}

+ (BOOL)gcdCallback{
    return _gcdCallback;
}

+ (BOOL)oaoaCallback{
    return _oaoaCallback;
}

+ (BOOL)udpCallback{
    return _udpCallback;
}

- (instancetype)initWithMessenger:(nonnull NSObject<FlutterBinaryMessenger> *)messenger {
    self = [super init];
    if (self) {
        // _streamHandler = [[AppcarryStreamHandler alloc] init];
        _callbackChannel = [FlutterMethodChannel methodChannelWithName:acCallbacksMethodChannel binaryMessenger:messenger];
        _eventChannel = [FlutterEventChannel eventChannelWithName:acEventChannel binaryMessenger:messenger];
    }
    return self;
}

+ (void)registerWithRegistrar:(NSObject<FlutterPluginRegistrar>*)registrar {
    id<FlutterBinaryMessenger> messenger = [registrar messenger];
    FlutterMethodChannel *channel = [FlutterMethodChannel methodChannelWithName:acMethodChannel binaryMessenger:messenger];
    FlutterMethodChannel *callbackChannel = [FlutterMethodChannel methodChannelWithName:acCallbacksMethodChannel binaryMessenger:messenger];
    AppcarrySdkPlugin *instance = [[AppcarrySdkPlugin alloc] initWithMessenger:messenger];
    [registrar addMethodCallDelegate:instance channel:channel];
    [registrar addMethodCallDelegate:instance channel:callbackChannel];
    [registrar addApplicationDelegate:instance];
}

- (void)handleMethodCall:(FlutterMethodCall*)call result:(FlutterResult)result {
    
    if([@"initSdk" isEqualToString:call.method]){
        [self initSdkWithCall:call result:result];
    }else if([@"getSDKVersion" isEqualToString:call.method]){
        [self getSDKVersion:result];
    }else if([@"logEvent" isEqualToString:call.method]){
        [self logEventWithCall:call result:result];
    }else if([@"setUserEmails" isEqualToString:call.method]){
        [self setUserEmails:call result:result];
    }else if([@"setAppUserId" isEqualToString:call.method]){
        [self setAppUserId:call result:result];
    }else if([@"setCurrencyCode" isEqualToString:call.method]){
        [self setCurrencyCode:call result:result];
    }else if([@"setWebHookURL" isEqualToString:call.method]){
        [self setWebHookURL:call result:result];
    }else if([@"Stop" isEqualToString:call.method]){
        [self Stop:call result:result];
    }else if([@"setHost" isEqualToString:call.method]){
        [self setHost:call result:result];
    }else if([@"setAdditionalData" isEqualToString:call.method]){
        [self setAdditionalData:call result:result];
    }else if([@"getAppcarryUID" isEqualToString:call.method]){
        [self getAppcarryUID:result];
    }else if([@"setSharingFilter" isEqualToString:call.method]){
        [self setSharingFilter:call result:result];
    }else if([@"setSharingFilterForAllPartners" isEqualToString:call.method]){
        [self setSharingFilterForAllPartners:result];
    }else if([@"setAppInviteLinkTrackID" isEqualToString:call.method]){
        [self setAppInviteLinkTrackID:call result:result];
    }else if([@"setLinkTrackCustomDomains" isEqualToString:call.method]){
        [self setLinkTrackCustomDomains:call result:result];
    }else if([@"disableSKAD" isEqualToString:call.method]){
        [self disableSKAD:call result:result];
    }else{
        result(FlutterMethodNotImplemented);
    }
}

- (void)disableSKAD:(FlutterMethodCall*)call result:(FlutterResult)result{
    id isSKADEnabled = call.arguments;
    if ([isSKADEnabled isKindOfClass:[NSNumber class]]) {
        _isSKADEnabled = [(NSNumber*)isSKADEnabled boolValue];
        [Appcarry getInstance].disableSKAdNetwork = _isSKADEnabled;
    }
    result(nil);
}

- (void)setLinkTrackCustomDomains:(FlutterMethodCall*)call result:(FlutterResult)result{
    NSArray* brandDomains = call.arguments;
    [[Appcarry getInstance] setLinkTrackCustomDomains:brandDomains];
    result(nil);
}

- (void)setAppInviteLinkTrackID:(FlutterMethodCall*)call result:(FlutterResult)result{
    NSString* appInviteLinkTrackID = call.arguments[@"inviteLinkTrackID"];
    [Appcarry getInstance].appInviteLinkTrackID = appInviteLinkTrackID;
    if([_callbackById containsObject:@"setAppInviteLinkTrackIDCallback"]){
        // NSDictionary* message = @{
        //     @"status": acSuccess
        // };
        // [_streamHandler sendResponseToFlutter:acAppInviteOneLinkID status:acSuccess data:message];
    }
    result(nil);
}

- (void)setSharingFilter:(FlutterMethodCall*)call result:(FlutterResult)result{
    NSArray* filters = call.arguments;
    [[Appcarry getInstance] setSharingFilter:filters];
    result(nil);
}

- (void)setSharingFilterForAllPartners:(FlutterResult)result{
    [[Appcarry getInstance] setSharingFilterForAllPartners];
    result(nil);
}

- (void)getAppcarryUID:(FlutterResult)result{
    result([[Appcarry getInstance] getAppcarryUID]);
}

- (void)getSDKVersion:(FlutterResult)result{
    result([[Appcarry getInstance] getSdkVersion]);
}

- (void)setHost:(FlutterMethodCall*)call result:(FlutterResult)result{
    NSString* hostName = call.arguments[@"hostName"];
    NSString* hostPrefix = call.arguments[@"hostPrefix"];
    [[Appcarry getInstance] setHost:hostName withHostPrefix:hostPrefix];
    result(nil);
}

- (void)setAppUserId:(FlutterMethodCall*)call result:(FlutterResult)result{
    NSString* userId = call.arguments[@"id"];
    [Appcarry getInstance].customerUserID  = userId;
    result(nil);
}

- (void)setCurrencyCode:(FlutterMethodCall*)call result:(FlutterResult)result{
    NSString* currencyId = call.arguments[@"currencyCode"];
    [Appcarry getInstance].currencyCode = currencyId;
    result(nil);
}

- (void)setWebHookURL:(FlutterMethodCall*)call result:(FlutterResult)result{
    NSString* webHookURL = call.arguments[@"webHookURL"];
    [Appcarry getInstance].webHookUrl = webHookURL;
    result(nil);
}

- (void)setAdditionalData:(FlutterMethodCall*)call result:(FlutterResult)result{
    NSDictionary* data = call.arguments[@"customData"];
    [[Appcarry getInstance] setAdditionalData:data];
    result(nil);
}

- (void)Stop:(FlutterMethodCall*)call result:(FlutterResult)result{
    BOOL stop = call.arguments[@"isStopped"];
    [Appcarry getInstance].isStopped  = stop;
    result(nil);
}

- (void)setUserEmails:(FlutterMethodCall*)call result:(FlutterResult)result{
    NSMutableArray *emails = call.arguments[@"emails"];
    NSArray *emaillsArray = [emails copy];
    NSNumber* cryptTypeInt = (id)call.arguments[@"cryptType"];
    
    EmailCryptType cryptType = EmailCryptTypeNone;
    if(1 == [cryptTypeInt doubleValue]){
        cryptType = EmailCryptTypeSHA256;
    }
    
    [[Appcarry getInstance] setUserEmails:emaillsArray withCryptType: cryptType];
    result(nil);
}

-(void)sendLaunch:(UIApplication *)application {
    [[Appcarry getInstance] start];
}

-(void)sendBackground:(UIApplication *)application {
    [[Appcarry getInstance] startBackground];
}

- (void)initSdkWithCall:(FlutterMethodCall*)call result:(FlutterResult)result{
 
    NSString* devKey = nil;
    NSString* appId = nil;
    // NSString* appInviteOneLink = nil;
    // BOOL disableCollectASA = NO;
    // BOOL disableAdvertisingIdentifier = NO;
    NSTimeInterval timeToWaitForATTUserAuthorization = 0;
    BOOL isDebug = NO;
    BOOL isConversionData = NO;
    // BOOL isUDP = NO;
    
    id isDebugValue = nil;
    id isConversionDataValue = nil;
    // id isUDPValue = nil;
    // id isDisableCollectASA = nil;
    // id isDisableAdvertisingIdentifier = nil;
    
    devKey = call.arguments[acDevKey];
    appId = call.arguments[acAppId];
    timeToWaitForATTUserAuthorization = [(id)call.arguments[acTimeToWaitForATTUserAuthorization] doubleValue];
    
    isDebugValue = call.arguments[acIsDebug];
    if ([isDebugValue isKindOfClass:[NSNumber class]]) {
        // isDebug is a boolean that will come through as an NSNumber
        isDebug = [(NSNumber*)isDebugValue boolValue];
    }
    isConversionDataValue = call.arguments[acConversionData];
    if ([isConversionDataValue isKindOfClass:[NSNumber class]]) {
        isConversionData = [(NSNumber*)isConversionDataValue boolValue];
    }
    if (isConversionData == YES) {
        //   [Appcarry getInstance].delegate = self;
    }
    
    // isUDPValue = call.arguments[acUDL];
    // if ([isUDPValue isKindOfClass:[NSNumber class]]) {
    //     isUDP = [(NSNumber*)isUDPValue boolValue];
    //     if(isUDP == YES){
    //         [AppsFlyerLib shared].deepLinkDelegate = _streamHandler;
    //     }
    // }
    
    // appInviteOneLink = call.arguments[acInviteOneLink];
    // if(appInviteOneLink != nil){
    //     [AppsFlyerLib shared].appInviteOneLinkID = appInviteOneLink;
    // }
    
    // isDisableCollectASA = call.arguments[acDisableCollectASA];
    // if ([isDisableCollectASA isKindOfClass:[NSNumber class]]) {
    //     // isDebug is a boolean that will come through as an NSNumber
    //     disableCollectASA = [(NSNumber*)isDisableCollectASA boolValue];
    // }
    // isDisableAdvertisingIdentifier = call.arguments[acDisableAdvertisingIdentifier];
    // if ([isDisableAdvertisingIdentifier isKindOfClass:[NSNumber class]]) {
    //     // isDebug is a boolean that will come through as an NSNumber
    //     disableAdvertisingIdentifier = [(NSNumber*)isDisableAdvertisingIdentifier boolValue];
    // }
    
    
    // [AppsFlyerLib shared].disableCollectASA = disableCollectASA;
    
    // SEL DisableAdvertisingSel = NSSelectorFromString(@"setDisableAdvertisingIdentifier:");
    // id AppsFlyer = [AppsFlyerLib shared];
    // if ([AppsFlyer respondsToSelector:DisableAdvertisingSel] && disableAdvertisingIdentifier) {
    //     bypassDisableAdvertisingIdentifier msgSend = (bypassDisableAdvertisingIdentifier)objc_msgSend;
    //     msgSend(AppsFlyer, DisableAdvertisingSel, disableAdvertisingIdentifier);
    // }
    
    [Appcarry getInstance].appleAppID = appId;
    [Appcarry getInstance].appcarryDevKey = devKey;
    [Appcarry getInstance].isDebug = isDebug;
    [Appcarry getInstance].platform_extension = @"ios_flutter";
    
    
    // SEL WaitForATTSel = NSSelectorFromString(@"waitForATTUserAuthorizationWithTimeoutInterval:");
    
    // if ([AppsFlyer respondsToSelector:WaitForATTSel] && timeToWaitForATTUserAuthorization != 0) {
    //     bypassWaitForATTUserAuthorization msgSend = (bypassWaitForATTUserAuthorization)objc_msgSend;
    //     msgSend(AppsFlyer, WaitForATTSel, timeToWaitForATTUserAuthorization);
    // }
    
    if (timeToWaitForATTUserAuthorization != 0) {
        [[Appcarry getInstance] waitForATTUserAuthorizationWithTimeoutInterval:timeToWaitForATTUserAuthorization];
    }
    
    [[Appcarry getInstance] start];

    //post notification for the deep link object that the bridge is set and he can handle deep link
    // [AppsFlyerAttribution shared].isBridgeReady = YES;
    // [[NSNotificationCenter defaultCenter] postNotificationName:AF_BRIDGE_SET object:self];
    
    [[NSNotificationCenter defaultCenter] addObserver:self
                                             selector:@selector(sendLaunch:)
                                                 name:UIApplicationDidBecomeActiveNotification
                                               object:nil];
    [[NSNotificationCenter defaultCenter] addObserver:self
                                             selector:@selector(sendBackground:)
                                                 name:UIApplicationDidEnterBackgroundNotification
                                               object:nil];    
    result(@{@"status": @"OK"});
}

-(void)logEventWithCall:(FlutterMethodCall*)call result:(FlutterResult)result{
    NSString *eventName =  call.arguments[acEventName];
    NSDictionary *eventValues = call.arguments[acEventValues];
    
    [[Appcarry getInstance] logEvent:eventName withValues:eventValues];
    //TODO: Add callback handler
    result(@YES);
}

+ (FlutterViewController*) getViewController{
    UIViewController *topMostViewControllerObj =  [[[UIApplication sharedApplication] delegate] window].rootViewController;
    FlutterViewController *flutterViewController = (FlutterViewController *)topMostViewControllerObj;
    
    return flutterViewController;
}

-(void) handleCallback:(NSArray *) objArray{
    NSDictionary* message = [objArray objectAtIndex:0];
    //NSString* channel = [objArray objectAtIndex:1];
    
    NSError *error;
    NSData *dataFromDict = [NSJSONSerialization dataWithJSONObject:message
                                                           options:NSJSONWritingPrettyPrinted
                                                             error:&error];
    [[NSNotificationCenter defaultCenter] postNotificationName:@"ac-events" object:dataFromDict];
    //if(!error){
    //[flutterViewController sendOnChannel:channel message:dataFromDict binaryReply:^(NSData * _Nullable reply) {
    //
    //}];
    //}
}

@end
